/*
 * libipv6addr: Function libary for IPv6 storage
 *
 * Version:		$Id: libipv6addr.c,v 0.26 2001/05/01 $
 * 
 * Author:		Peter Bieringer <pb@bieringer.de>
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "libipv6addr.h"


/* function returns a word of an IPv6 address
 *
 * in: ipv6addrp = pointer to IPv6 address structure
 * in: numword   = number of word (0 = MSB, 7 = LSB)
 * additional: calls exit on out of range
 */
#define DEBUG_function_name "libipv6addr/ipv6addr_getword"
unsigned int ipv6addr_getword(ipv6calc_ipv6addr *ipv6addrp, int numword) {
	unsigned int retval;
	
	if ( ( numword < 0 ) || ( numword > 7 ) ) {
		fprintf(stderr, "%s: given word number '%d' is out of range!\n", DEBUG_function_name, numword);
		exit(2);
	};

	retval = ( ipv6addrp->in6_addr.s6_addr[numword * 2] << 8 ) \
		| ( ipv6addrp->in6_addr.s6_addr[numword * 2 + 1] );

	return (retval);
};
#undef DEBUG_function_name


/* function returns a dword of an IPv6 address
 *
 * in: ipv6addrp = pointer to IPv6 address structure
 * in: numdword  = number of word (0 = MSB, 3 = LSB)
 * additional: calls exit on out of range
 */
#define DEBUG_function_name "libipv6addr/ipv6addr_getdword"
unsigned int ipv6addr_getdword(ipv6calc_ipv6addr *ipv6addrp, int numdword) {
	unsigned int retval;
	
	if ( ( numdword < 0 ) || ( numdword > 3 ) ) {
		fprintf(stderr, "%s: given dword number '%d' is out of range!\n", DEBUG_function_name, numdword);
		exit(2);
	};

	retval = ( ipv6addrp->in6_addr.s6_addr[numdword * 4] << 24 ) \
		| ( ipv6addrp->in6_addr.s6_addr[numdword * 4 + 1] << 16 ) \
		| ( ipv6addrp->in6_addr.s6_addr[numdword * 4 + 2] << 8 ) \
		| ( ipv6addrp->in6_addr.s6_addr[numdword * 4 + 3] ); 

	return (retval);
};
#undef DEBUG_function_name


/* function sets a word of an IPv6 address
 *
 * mod: ipv6addrp = pointer to IPv6 address structure
 * in: numword   = number of word (0 = MSB, 7 = LSB)
 * in: value     = value to set
 * additional: calls exit on out of range
 */
#define DEBUG_function_name "libipv6addr/ipv6addr_setword"
void ipv6addr_setword(ipv6calc_ipv6addr *ipv6addrp, int numword, unsigned int value) {
	
	if ( ( numword < 0 ) || ( numword > 7 ) ) {
		fprintf(stderr, "%s: given word number '%d' is out of range!\n", DEBUG_function_name, numword);
		exit(2);
	};
	
	if ( ( value < 0 ) || ( value > 0x0000ffff ) ) {
		fprintf(stderr, "%s: given value '%x' is out of range!\n", DEBUG_function_name, value);
		exit(2);
	}; 

	ipv6addrp->in6_addr.s6_addr[numword * 2    ] = ( value & 0x0000ff00 ) >>  8;
	ipv6addrp->in6_addr.s6_addr[numword * 2 + 1] = ( value & 0x000000ff )      ;

	return;
};
#undef DEBUG_function_name


/* function sets a dword of an IPv6 address
 *
 * mod: ipv6addrp = pointer to IPv6 address structure
 * in: numdword  = number of word (0 = MSB, 3 = LSB)
 * in: value     = value to set
 * additional: calls exit on out of range
 */
#define DEBUG_function_name "libipv6addr/ipv6addr_setdword"
void ipv6addr_setdword(ipv6calc_ipv6addr *ipv6addrp, int numdword, unsigned int value) {
	
	if ( ( numdword < 0 ) || ( numdword > 3 ) ) {
		fprintf(stderr, "%s: given dword number '%d' is out of range!\n", DEBUG_function_name, numdword);
		exit(2);
	};

	if ( ( value < 0 ) || ( value > 0xffffffff ) ) {
		fprintf(stderr, "%s: given value '%x' is out of range!\n", DEBUG_function_name, value);
		exit(2);
	}; 

	ipv6addrp->in6_addr.s6_addr[numdword * 4    ] = ( value & 0xff000000 ) >> 24;
	ipv6addrp->in6_addr.s6_addr[numdword * 4 + 1] = ( value & 0x00ff0000 ) >> 16;
	ipv6addrp->in6_addr.s6_addr[numdword * 4 + 2] = ( value & 0x0000ff00 ) >>  8;
	ipv6addrp->in6_addr.s6_addr[numdword * 4 + 3] = ( value & 0x000000ff )      ;

	return;
};
#undef DEBUG_function_name


/* function clears the IPv6 structure
 *
 * mod: ipv6addrp = pointer to IPv6 address structure
 */
#define DEBUG_function_name "libipv6addr/ipv6addr_clear"
void ipv6addr_clear(ipv6calc_ipv6addr *ipv6addrp) {
	int i;

	for (i = 0; i <= 15; i++) {
		ipv6addrp->in6_addr.s6_addr[i] = 0;
	};
	
	return;
};
#undef DEBUG_function_name
